#!/usr/bin/env perl
#!/bin/env perl

## Name: posnegcomparechr.pl
## Version: 1.2.4
## Date: Apr 21, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

## determine the relative position of new transcription units w.r.t. inversely-stranded transcription units   
# --input, input gff file
# --reference, reference gff file
# --output, output prefix

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $reference;
my $output = "nc";
my $linenew;
my $lineref;
my @argsref;
my @argsnew;
my @argsprev;
my @noteref;
my @noteprev;

usage() if (@ARGV < 4 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "reference=s" => \$reference,
			 "output=s" => \$output)
	    or defined $help);

open(REF, $reference) or die("Reference file not found\n");
open(NEW, $input) or die("Input file not found\n");

open(INT, ">$output.intergenic.gff");
open(OVL, ">$output.overlapped.gff");
open(INC, ">$output.included.gff");

$argsnew[4] = -1;

 WHOLE: {
     while ($lineref = <REF>) {
	 chomp($lineref);
	 @argsref = split("\t", $lineref);
	 @noteref = split(";", $argsref[8]);#exons=

	 #< INT: all new transcripts coming before the current reference 
	 while ($argsnew[4] < $argsref[3]) {
	     if ($argsnew[4] > -1) {
		 print INT "$linenew; ";
		 if ($#argsprev>0) {
		     @noteprev = split(";", $argsprev[8]);#exons=
		     print INT "leftanti=$noteprev[0]; rightanti=$noteref[0]\n";
		 }
		 else {
		     print INT "rightanti=$noteref[0]\n";
		 }
	     }
	     $linenew = <NEW>;
	     if (!$linenew) {
		 last WHOLE;
	     }
	     chomp($linenew);
	     @argsnew = split("\t", $linenew);
	 }
	 #>
	 
	 #< OVL: all new transcripts overlapping with the current reference
	 while ($argsnew[3] < $argsref[3]) {
	     print OVL "$linenew; overlap=$noteref[0]\n";
	     $linenew = <NEW>;
	     if (!$linenew) {
		 last WHOLE;
	     }
	     chomp($linenew);
	     @argsnew = split("\t", $linenew);
	 }
	 #>

	 #< INC: all new transcripts included in the current reference
	 while ($argsnew[4] <= $argsref[4]) {
	     print INC "$linenew; include=$noteref[0]\n";
	     $linenew = <NEW>;
	     if (!$linenew) {
		 last WHOLE;
	     }
	     chomp($linenew);
	     @argsnew = split("\t", $linenew);
	 }
	 #>
	 
	 #< OVL: all new transcripts overlapping with the current reference
	 while ($argsnew[3] < $argsref[4]) {
	     print OVL "$linenew; overlap=$noteref[0]\n";
	     $linenew = <NEW>;
	     if (!$linenew) {
		 last WHOLE;
	     }
	     chomp($linenew);
	     @argsnew = split("\t", $linenew);
	 }
	 #>
	 
	 @argsprev = @argsref;
     }
}

#< transcripts coming after the last reference
do {
    if ($linenew) {
	print INT "$linenew; ";
	if ($#argsprev>0) {
	    @noteprev = split(";", $argsprev[8]);
	    print INT "leftanti=$noteprev[0]";
	}
	print INT "\n";
    }
    $linenew = <NEW>;
    if ($linenew) {
	chomp($linenew);
    }
} until (!$linenew && eof(NEW));
#>

close(REF);
close(NEW);
close(INT);
close(OVL);
close(INC);
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl posnegcomparechr.pl --input GFF --reference REF [--output OUT_PREFIX] [--help|?] \n";
    exit 1;
}
