#!/usr/bin/env perl
#!/bin/env perl

## Name: separatetscexon.pl
## Version: 1.2.4
## Date: Apr 21, 2016
## Author: Van Du Tran and Daniel Gautheret
## Maintainer: Van Du Tran and Daniel Gautheret

# separate transcript list and exon list, giving input for profile comparison
# --input, input gff file, containing only transcripts and their exons
##e.g.
##transcript
##exon
##exon
##transcript
##exon
# --transcript, gff transcript output file
# --exon, gff exon output file

use strict;
use warnings;
use Getopt::Long;

my $help;
my $input;
my $transcript = "transcripts.gff";
my $exon;
my $line;
my @args;
my @tscargs;
my @exons;

usage() if (@ARGV < 2 or
	    ! GetOptions("help|?" => \$help,
			 "input=s" => \$input,
			 "transcript=s" => \$transcript,
			 "exon=s" => \$exon)
	    or defined $help);

open(GFF, $input) or die("Input file not found\n");
open(TSC, ">$transcript");

$line = <GFF>;
if ($line) {
    chomp($line);
}
else {
    print "Empty input file\n";
    exit 0;
}
do {
    #< take transcript
    print TSC "$line; exons=";
    @tscargs = split("\t", $line);
    #>
    $line = <GFF>;
    if ($line) {
	chomp($line);
	@args = split("\t", $line);
    }

    #< if a transcript does not have exons which follow, then exon=transcript
    if ($args[2] ne "exon") {
	print TSC "$tscargs[3] $tscargs[4] ";
    }
    #>
    
    @exons = ();
    #< put exons to transcript note
    while ($line && $args[2] eq "exon") {
     	if ($#exons < 0 || $exons[$#exons] < $args[3]) {
	    push(@exons, $args[3], $args[4]);
	}
	else {
	    unshift(@exons, $args[3], $args[4]);
	}
	$line = <GFF>;
	if ($line) {
	    chomp($line);
	    @args = split("\t", $line);
	}
    }
    #>
    print TSC "@exons\n";
} until (!$line);
close(GFF);
close(TSC);

#<sort transcript file
system("sort -k1,1 -k4,4n -k5,5n $transcript > $transcript.sort");
#>

if (!$exon) {
    system("mv $transcript.sort $transcript");
}
else {
    open(SORT, "$transcript.sort");
    open(TSC, ">$transcript");
    open(EXON, ">$exon");
    
    while ($line = <SORT>) {
	chomp($line);
	@args = split("exons=", $line);
	print TSC "$args[0]\n";
	print EXON "$args[1]\n";
    }
    
    close(SORT);
    close(TSC);
    close(EXON);
}
exit 0;

sub usage {
    print "Unknown option: @_\n" if ( @_ );
    print "Usage: perl separatetscexon.pl --input GFF [--transcript TSC] [--exon EXO] [--help|?] \n";
    print "Notice: input file should only contain transcript and exon features\n";
    exit 1;
}
